{ *********************************************************************** }
{                                                                         }
{ Adit Math Engine                                                        }
{ Math Edit Demo                                                          }
{                                                                         }
{ Copyright (c) 2015-21 Adit Software http://www.aditmath.com/            }
{                                                                         }
{ *********************************************************************** }


unit frmMain;

{.$GENERATELABGFILES}

interface

uses
  Winapi.Windows, Winapi.Messages,
  System.Types, System.SysUtils, System.Variants, System.Classes, System.Math,
  System.Actions,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Grids,
  Vcl.Themes, Vcl.ComCtrls, Vcl.ExtCtrls, Vcl.Clipbrd, Vcl.Menus, Vcl.Buttons,
  Vcl.ActnList, Xml.Win.msxmldom,
  MathBase, MathTable, MathParser, MathPresentation, MathEquation, MathScroll,
  MathEdit, MathActions, MathManager, MathComponent, AdSymbolsGrid, MathLocalize,
  MathLoc_EngUS, MathLoc_Rus, MathSymbols, dmAdActions;

type
  TMainForm = class(TForm, IMathLocalize)
    pgcMain: TPageControl;
    tsSymbols: TTabSheet;
    pnlClient: TPanel;
    pnlSymbolsTop: TPanel;
    cbFilter: TComboBox;
    splPc: TSplitter;
    tsExpr: TTabSheet;
    Panel1: TPanel;
    cbExpr: TComboBox;
    cd: TColorDialog;
    mcEdit: TMathEditControl;
    pmEdit: TMathEditPopupMenu;
    egSymbols: TAdExpressionGrid;
    egExpression: TAdExpressionGrid;
    mmMain: TMainMenu;
    miFile: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    Export1: TMenuItem;
    miEdit: TMenuItem;
    Undo1: TMenuItem;
    Redo1: TMenuItem;
    N1: TMenuItem;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    PasteasText1: TMenuItem;
    N2: TMenuItem;
    Group1: TMenuItem;
    Ungroup1: TMenuItem;
    N3: TMenuItem;
    Delete1: TMenuItem;
    SelectAll1: TMenuItem;
    miFont: TMenuItem;
    miInsert: TMenuItem;
    miOptions: TMenuItem;
    Space1: TMenuItem;
    FigureSpace1: TMenuItem;
    PunctuationSpace1: TMenuItem;
    AlignmentPoint1: TMenuItem;
    ManualBreak1: TMenuItem;
    DisplayLineBreak1: TMenuItem;
    SoftBreak1: TMenuItem;
    Space2: TMenuItem;
    N4: TMenuItem;
    Symbol1: TMenuItem;
    Rule1: TMenuItem;
    Matrix1: TMenuItem;
    Zoom1: TMenuItem;
    EquationProperties1: TMenuItem;
    NonprintingCharacters1: TMenuItem;
    miDisplayStyle: TMenuItem;
    DisplayStyle1: TMenuItem;
    InlineStyle1: TMenuItem;
    InkEquation1: TMenuItem;
    Font2: TMenuItem;
    ClearFormatting1: TMenuItem;
    N5: TMenuItem;
    miStyle: TMenuItem;
    miVariant: TMenuItem;
    N6: TMenuItem;
    FontColor1: TMenuItem;
    HyperlinkFontColor1: TMenuItem;
    Background1: TMenuItem;
    Bold1: TMenuItem;
    Italic1: TMenuItem;
    BoldFraktur1: TMenuItem;
    BoldItalic1: TMenuItem;
    Bold2: TMenuItem;
    BoldScript1: TMenuItem;
    ResettoDefaultBold1: TMenuItem;
    ResettoDefault1: TMenuItem;
    DoubleStruck1: TMenuItem;
    Fraktur1: TMenuItem;
    Italic2: TMenuItem;
    Monospace1: TMenuItem;
    Normal1: TMenuItem;
    BoldItalicSansSerif1: TMenuItem;
    BoldSansSerif1: TMenuItem;
    ItalicSansSerif1: TMenuItem;
    SansSerif1: TMenuItem;
    N7: TMenuItem;
    N8: TMenuItem;
    Script1: TMenuItem;
    N9: TMenuItem;
    N10: TMenuItem;
    sbMain: TStatusBar;
    miLanguages: TMenuItem;
    miEnglish: TMenuItem;
    miRussian: TMenuItem;
    cbFilterEx: TComboBox;
    cbExprEx: TComboBox;
    miHelp: TMenuItem;
    miAbout: TMenuItem;
    miExit: TMenuItem;
    N11: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure cbFilterChange(Sender: TObject);
    procedure cbExprChange(Sender: TObject);
    procedure TextOnHint(Sender: TObject);
    procedure miLanguagesClick(Sender: TObject);
    procedure miEnglishClick(Sender: TObject);
    procedure miRussianClick(Sender: TObject);
    procedure cbFilterExChange(Sender: TObject);
    procedure cbExprExChange(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure miAboutClick(Sender: TObject);
  public
    procedure Localize;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

type
  TMathSaveActionAccess = class (TMathSaveAction);

const
  WelcomeText = '\operatorname {erf} (z)={\frac {2}{\sqrt {\pi }}}\sum _{n=0}^{\infty }' +
    '\left(z\prod _{k=1}^{n}{\frac {-(2k-1)z^{2}}{k(2k+1)}}\right)={\frac {2}{\sqrt {\pi }}}' +
    '\sum _{n=0}^{\infty }{\frac {z}{2n+1}}\prod _{k=1}^{n}{\frac {-z^{2}}{k}}';

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := TMathSaveActionAccess(AdActionsModule.MathSaveAction1).SaveDocEx(mcEdit);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  mcEdit.CopyFormats := mcEdit.CopyFormats + [mcfSVG];

  Xml.Win.msxmldom.MSXMLDOMDocumentFactory.AddDOMProperty('ProhibitDTD', False);
  mcEdit.LoadFromLatex('', WelcomeText);
  mcEdit.CreateMathPanelIfPossible;
  Application.OnHint := TextOnHint;
  Localize;
end;

procedure TMainForm.miLanguagesClick(Sender: TObject);
begin
  miEnglish.Checked := GetCurrentLanguage is TMathEngUSLanguage;
  miRussian.Checked := GetCurrentLanguage is TMathRusLanguage;
end;

procedure TMainForm.Localize;
var
  I: TMathSymbolFilterType;
  J: TMathExpressionFilterType;
  Index: integer;
begin
  Index := cbFilter.ItemIndex;
  cbFilter.Items.Clear;
  for I := Low(TMathSymbolFilterType) to High(TMathSymbolFilterType) do
    cbFilter.Items.AddObject(GetMathSymbolFilterTypeStr(I), TObject(I));
  if Index >= 0 then
    cbFilter.ItemIndex := Index
  else
    cbFilter.ItemIndex := 0;
  cbFilterChange(nil);

  Index := cbExpr.ItemIndex;
  cbExpr.Items.Clear;
  for J := Low(TMathExpressionFilterType) to High(TMathExpressionFilterType) do
    cbExpr.Items.AddObject(GetMathExpressionFilterTypeStr(J), TObject(J));
  if Index >= 0 then
    cbExpr.ItemIndex := Index
  else
    cbExpr.ItemIndex := 0;
  cbExprChange(nil);

  miFile.Caption := MathGetStr(math_menu_File);
  miEdit.Caption := MathGetStr(math_menu_Edit);
  miFont.Caption := MathGetStr(math_menu_Font);
  miInsert.Caption := MathGetStr(math_menu_Insert);
  miOptions.Caption := MathGetStr(math_menu_Tools);
  miHelp.Caption := MathGetStr(math_menu_Help);
  miVariant.Caption := MathGetStr(math_menu_ApplyStyle);
  miStyle.Caption := MathGetStr(math_menu_FontStyle);
  miDisplayStyle.Caption := MathGetStr(math_menu_StyleFormatting);
  miLanguages.Caption := MathGetStr(math_menu_Languages);
  miAbout.Caption := MathGetStr(math_menu_About);
  miExit.Caption := MathGetStr(math_menu_Exit);

  tsSymbols.Caption := MathGetStr(math_menu_Symbols);
  tsExpr.Caption := MathGetStr(math_menu_Structures);

  LocalizeMathFormAction(Self);
  LocalizeMathFormAction(AdActionsModule);
end;

procedure TMainForm.miAboutClick(Sender: TObject);
begin
  ShowMessage('Adit Math Engine Demo'#13#10'http://aditmath.com');
end;

procedure TMainForm.miEnglishClick(Sender: TObject);
{$IFDEF GENERATELABGFILES}
var
  SL: TStringList;
{$ENDIF}
begin
  SwitchLanguage(TMathEngUSLanguage);
  {$IFDEF GENERATELABGFILES}
  SL := GetCurrentLanguage.GetNormalizeList;
  try
	SL.SaveToFile('english.txt', TEncoding.UTF8);
  finally
	SL.Free;
  end;
  {$ENDIF}
end;

procedure TMainForm.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.miRussianClick(Sender: TObject);
{$IFDEF GENERATELABGFILES}
var
  SL: TStringList;
{$ENDIF}
begin
  SwitchLanguage(TMathRusLanguage);
  {$IFDEF GENERATELABGFILES}  
  SL := GetCurrentLanguage.GetNormalizeList;
  try
	SL.SaveToFile('russian.txt', TEncoding.UTF8);
  finally
	SL.Free;
  end;  
  {$ENDIF}
end;

procedure TMainForm.cbExprChange(Sender: TObject);
var
  FilterType: TMathExpressionFilterType;
  ExprTypes: TMathExpressionTypes;
  ExprType: TMathExpressionType;
begin
  if cbExpr.ItemIndex >= 0 then
  begin
    FilterType := TMathExpressionFilterType(cbExpr.Items.Objects[cbExpr.ItemIndex]);
    ExprTypes := GetExpressionFilterRange(FilterType);

    cbExprEx.Items.BeginUpdate;
    try
      cbExprEx.Items.Clear;
      for ExprType := Low(TMathExpressionType) to High(TMathExpressionType) do
        if ExprType in ExprTypes then
          cbExprEx.Items.AddObject(GetMathExpressionTypeStr(ExprType), TObject(ExprType));
      if cbExprEx.Items.Count > 0 then
        cbExprEx.ItemIndex := 0;
      cbExprEx.Enabled := cbExprEx.Items.Count > 1;
    finally
      cbExprEx.Items.EndUpdate;
      cbExprExChange(nil);
    end;
  end;
end;

procedure TMainForm.cbExprExChange(Sender: TObject);
var
  ExprType: TMathExpressionType;
begin
  if cbExprEx.ItemIndex >= 0 then
  begin
    ExprType := TMathExpressionType(cbExprEx.Items.Objects[cbExprEx.ItemIndex]);
    egExpression.FillExpressions([ExprType]);
  end;
end;

procedure TMainForm.cbFilterChange(Sender: TObject);
var
  FilterType: TMathSymbolFilterType;
  SymbolTypes: TMathSymbolTypes;
  SymbolType: TMathSymbolType;
begin
  if cbFilter.ItemIndex >= 0 then
  begin
    FilterType := TMathSymbolFilterType(cbFilter.Items.Objects[cbFilter.ItemIndex]);
    SymbolTypes := GetSymbolsFilterRange(FilterType);

    cbFilterEx.Items.BeginUpdate;
    try
      cbFilterEx.Items.Clear;
      for SymbolType := Low(TMathSymbolType) to High(TMathSymbolType) do
        if SymbolType in SymbolTypes then
          cbFilterEx.Items.AddObject(GetMathSymbolTypeStr(SymbolType), TObject(SymbolType));
      if cbFilterEx.Items.Count > 0 then
        cbFilterEx.ItemIndex := 0;
      cbFilterEx.Enabled := cbFilterEx.Items.Count > 1;
    finally
      cbFilterEx.Items.EndUpdate;
      cbFilterExChange(nil);
    end;
  end;
end;

procedure TMainForm.cbFilterExChange(Sender: TObject);
var
  SymbolType: TMathSymbolType;
begin
  if cbFilterEx.ItemIndex >= 0 then
  begin
    SymbolType := TMathSymbolType(cbFilterEx.Items.Objects[cbFilterEx.ItemIndex]);
    egSymbols.FillSymbols([SymbolType]);
  end;
end;

procedure TMainForm.TextOnHint(Sender: TObject);
begin
  sbMain.Panels[0].Text := GetLongHint(Application.Hint);
end;

end.
